"""
Configuration helpers for TAPPs. Config file and record management are here,
as well as logging and redis.
Database configuration is done in sqlalchemy_models, since it has the content to test with.
"""

import ConfigParser
import logging
import os
import redis

__all__ = ['get_config', 'setup_logging', 'setup_redis']


def get_config(name=__name__):
    """
    Get a configuration parser for a given TAPP name.
    Reads config.ini files only, not in-database configuration records.

    :param name: The tapp name to get a configuration for.
    :rtype: ConfigParser
    :return: A config parser matching the given name
    """
    cfg = ConfigParser.ConfigParser()
    path = os.environ.get('%s_CONFIG_FILE' % name.upper(), 'cfg.ini')
    cfg.read(path)
    return cfg


def setup_logging(cfg=None):
    """
    Create a logger, based on the given configuration.
    Accepts LOGFILE and LOGLEVEL settings.

    :param cfg: The configuration object with logging info.
    :return: The session and the engine as a list (in that order)
    """
    logfile = cfg.get('log', 'LOGFILE') if cfg is not None and \
        cfg.get('log', 'LOGFILE') is not None else 'server.log'
    loglevel = cfg.get('log', 'LOGLEVEL') if cfg is not None and \
        cfg.get('log', 'LOGLEVEL') is not None else logging.INFO
    logging.basicConfig(filename=logfile, level=loglevel)
    return logging.getLogger(__name__)


def setup_redis():
    """
    Setup redis for use in a TAPP. For now this is extremely dumb,
    and takes no configuration.

    :rtype: Strictredis
    :return: A redis connection to use.
    """
    red = redis.StrictRedis()
    return red



import logging
import os
import unittest
from redis import StrictRedis
from tapp_config import setup_redis, get_config, setup_logging


class TestRedis(unittest.TestCase):
    def test_setup_redis(self):
        red = setup_redis()
        assert isinstance(red, StrictRedis)
        red.set("testing", "1.2.3")
        test = red.get("testing")
        assert test == "1.2.3"


class TestGetConfig(unittest.TestCase):
    def setUp(self):
        if "TEST_CONFIG_FILE" in os.environ:
            del os.environ["TEST_CONFIG_FILE"]

    def tearDown(self):
        self.setUp()

    def test_get_default_config(self):
        cfg = get_config("test")
        assert cfg.get('db', 'SA_ENGINE_URI') == 'sqlite:////tmp/test.db'
        assert cfg.get('log', 'LOGFILE') == '/tmp/test.log'

    def test_get_environment_config(self):
        path = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'env_cfg.ini')
        os.environ["TEST_CONFIG_FILE"] = path
        cfg = get_config("test")
        assert cfg.get('db', 'SA_ENGINE_URI') == 'sqlite:////tmp/env_test.db'
        assert cfg.get('log', 'LOGFILE') == '/tmp/env_test.log'


class TestSetupLogger(unittest.TestCase):
    def setUp(self):
        try:
            os.remove("/tmp/*test.log")
        except OSError:
            pass
        try:
            os.remove("server.log")
        except OSError:
            pass

    def tearDown(self):
        self.setUp()

    def test_setup_default_logger(self):
        logger = setup_logging()
        assert isinstance(logger, logging.getLoggerClass())
        message = 'testing 1.2.3.'
        logger.exception(message)
        assert os.path.isfile('server.log')

    def test_setup_logger(self):
        cfg = get_config("test")
        assert cfg.get('log', 'LOGFILE') == '/tmp/test.log'
        logger = setup_logging(cfg)
        assert isinstance(logger, logging.getLoggerClass())
        message = 'testing 1.2.3.'
        logger.exception(message)
        assert os.path.isfile('/tmp/test.log')






from setuptools import setup

classifiers = [
    "License :: OSI Approved :: MIT License",
    "Programming Language :: Python :: 2",
    "Topic :: Software Development :: Libraries",
]

# Cut the readme before the second section start.
readme = open('./README.rst').read()
url = 'https://github.com/gitguild/tapp-config'
top = readme.find('Usage\n-----')
bottom = readme.find('.. |Build Status|')
more = ("Visit {} to see the full README, the\n"
        "issue tracker, and others.\n\n\n")
readme = readme[:top] + more.format(url) + readme[bottom:]

setup(
    name='tapp-config',
    version='0.0.1',
    py_modules=['tapp_config'],
    url=url,
    long_description=readme,
    license='MIT',
    classifiers=classifiers,
    author='Ira Miller',
    author_email='ira@gitguild.com',
    description='Configuration for tapps (redis, logger, etc.)',
    setup_requires=['pytest-runner'],
    install_requires=[
        'redis'
    ],
    tests_require=['pytest', 'pytest-cov']
)
